(*----------------------------------------------------------------------------*
 *  Direct3D sample from DirectX 9.0 SDK Summer 2003                          *
 *  Delphi adaptation by Alexey Barkovoy (e-mail: clootie@ixbt.com)           *
 *                                                                            *
 *  Delphi versions 5-7 are supported                                         *
 *                                                                            *
 *  Modified: 30-Dec-2003                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru/delphi                                         *
 *----------------------------------------------------------------------------*)
//-----------------------------------------------------------------------------
// File: Text3D.cpp
//
// Desc: Example code showing how to do text in a Direct3D scene.
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------

unit Text3DUnit;

interface

uses
  Windows, Messages, SysUtils, CommDlg, Resource,
  Direct3D9, D3DX9, D3DApp, D3DEnumeration, D3DSettings,
  D3DFile, D3DFont, D3DUtil, DXUtil;

type
  //-----------------------------------------------------------------------------
  // Name: class CMyD3DApplication
  // Desc: Application class. The base class (CD3DApplication) provides the
  //       generic functionality needed in all Direct3D samples. CMyD3DApplication
  //       adds functionality specific to this sample program.
  //-----------------------------------------------------------------------------
  CMyD3DApplication = class(CD3DApplication)
  protected
    m_pFont:         ID3DXFont;
    m_pFont2:        ID3DXFont;
    m_pStatsFont:    ID3DXFont;
    m_pMesh3DText:   ID3DXMesh;
    m_strTextBuffer: PWideChar;

    m_pD3DXSprite:   ID3DXSprite;

    m_Camera: CFirstPersonCamera;

    m_strFont: array[0..LF_FACESIZE-1] of Char;
    m_nFontSize: Integer;

    m_matObj1: TD3DXMatrixA16;
    m_matObj2: TD3DXMatrixA16;

  protected
    function OneTimeSceneInit: HRESULT; override;
    function InitDeviceObjects: HRESULT; override;
    function RestoreDeviceObjects: HRESULT; override;
    function InvalidateDeviceObjects: HRESULT; override;
    function DeleteDeviceObjects: HRESULT; override;
    function Render: HRESULT; override;
    function FrameMove: HRESULT; override;
    function FinalCleanup: HRESULT; override;
    function CreateD3DXTextMesh(out ppMesh: ID3DXMesh; pstrFont: PChar; dwSize: DWORD; bBold, bItalic: Boolean): HRESULT;
    function CreateD3DXFont(out ppd3dxFont: ID3DXFont; pstrFont: PChar; dwSize: DWORD): HRESULT;

  public
    constructor Create;
    function MsgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM ): LRESULT; override;
  end;

implementation


constructor CMyD3DApplication.Create;
begin
  inherited;
  m_Camera:= CFirstPersonCamera.Create;

  m_strWindowTitle   := 'Text3D: Text in a 3D scene';
  m_d3dEnumeration.AppUsesDepthBuffer := True;
  m_pMesh3DText      := nil;
  m_dwCreationWidth  := 640;
  m_dwCreationHeight := 480;

  // Create fonts
  m_strFont:= 'Arial';
  m_nFontSize   := 12;
  m_pFont       := nil;
  m_pFont2      := nil;
  m_pStatsFont  := nil;
  m_pD3DXSprite := nil;
  m_strTextBuffer := nil;
end;


//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit: HRESULT;
var
  rsrc: HRSRC;
  hgData: HGLOBAL;
  pvData: Pointer;
  cbData: DWORD;
  nSize: Integer;
begin
  rsrc := FindResource(0, MAKEINTRESOURCE(IDR_TXT), 'TEXT');
  if (rsrc <> 0) then
  begin
    cbData := SizeofResource(0, rsrc);
    if (cbData > 0) then
    begin
      hgData := LoadResource(0, rsrc);
      if (hgData <> 0) then
      begin
        pvData := LockResource(hgData);
        if (pvData <> nil) then
        begin
          nSize := cbData div SizeOf(WideChar) + 1;
          GetMem(m_strTextBuffer, SizeOf(WideChar)*nSize);
          CopyMemory(m_strTextBuffer, pvData, cbData);
          m_strTextBuffer[nSize-1] := #0;
        end;
      end;
    end;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.InitDeviceObjects: HRESULT;
var
  nHeight: Integer;
  hDC: Windows.HDC;
  nLogPixelsY: Integer;
begin
  hDC := GetDC(0);
  nLogPixelsY := GetDeviceCaps(hDC, LOGPIXELSY);
  ReleaseDC(0, hDC);

  nHeight := -m_nFontSize * nLogPixelsY div 72;
  {hr= }D3DXCreateFont(m_pd3dDevice,          // D3D device
                       nHeight,               // Height
                       0,                     // Width
                       FW_BOLD,               // Weight
                       0,                     // MipLevels, 0 = autogen mipmaps
                       FALSE,                 // Italic
                       DEFAULT_CHARSET,       // CharSet
                       OUT_DEFAULT_PRECIS,    // OutputPrecision
                       DEFAULT_QUALITY,       // Quality
                       DEFAULT_PITCH or FF_DONTCARE, // PitchAndFamily
                       m_strFont,              // pFaceName
                       m_pFont);              // ppFont

  nHeight := -9 * nLogPixelsY div 72;
  Result := D3DXCreateFont(m_pd3dDevice, nHeight, 0, FW_BOLD, 0, FALSE, DEFAULT_CHARSET,
                           OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH or FF_DONTCARE,
                           'System', m_pFont2);
  if FAILED(Result) then Exit;

  nHeight := -9 * nLogPixelsY div 72;
  Result := D3DXCreateFont(m_pd3dDevice, nHeight, 0, FW_BOLD, 0, FALSE, DEFAULT_CHARSET,
                           OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH or FF_DONTCARE,
                           'Arial', m_pStatsFont);
  if FAILED(Result) then Exit;

  Result := D3DXCreateSprite(m_pd3dDevice, m_pD3DXSprite);
  if FAILED(Result) then Exit;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: CreateD3DXFont()
// Desc: 
//-----------------------------------------------------------------------------
function CMyD3DApplication.CreateD3DXFont(out ppd3dxFont: ID3DXFont;
  pstrFont: PChar; dwSize: DWORD): HRESULT;
var
  pd3dxFontNew: ID3DXFont;
  hDC_: HDC;
  nHeight: Integer;
begin
  hDC_ := GetDC(0);
  nHeight := -MulDiv(dwSize, GetDeviceCaps(hDC_, LOGPIXELSY), 72);
  ReleaseDC(0, hDC_);


  Result := D3DXCreateFont(m_pd3dDevice,          // D3D device
                           nHeight,               // Height
                           0,                     // Width
                           FW_BOLD,               // Weight
                           0,                     // MipLevels
                           FALSE,                 // Italic
                           DEFAULT_CHARSET,       // CharSet
                           OUT_DEFAULT_PRECIS,    // OutputPrecision
                           DEFAULT_QUALITY,       // Quality
                           DEFAULT_PITCH or FF_DONTCARE, // PitchAndFamily
                           pstrFont,              // pFaceName
                           pd3dxFontNew);         // ppFont

  if SUCCEEDED(Result) then ppd3dxFont := pd3dxFontNew;
end;


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects: HRESULT;
var
  light: TD3DLight9;
  vecLightDirUnnormalized: TD3DXVector3;
  matWorld: TD3DXMatrix;
  vecEye, vecAt: TD3DXVector3;
  fAspectRatio: Single;
begin
  // Restore the fonts
  m_pStatsFont.OnResetDevice;
  m_pFont.OnResetDevice;
  m_pFont2.OnResetDevice;
  m_pD3DXSprite.OnResetDevice;

  // Restore the textures
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  m_pd3dDevice.SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
  m_pd3dDevice.SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);

  m_pd3dDevice.SetRenderState(D3DRS_DITHERENABLE, 1);
  m_pd3dDevice.SetRenderState(D3DRS_SPECULARENABLE, 1);
  m_pd3dDevice.SetRenderState(D3DRS_LIGHTING, iTrue);
  m_pd3dDevice.SetRenderState(D3DRS_AMBIENT, $80808080);
  vecLightDirUnnormalized := D3DXVector3(10.0, -10.0, 10.0);
  ZeroMemory(@light, SizeOf(TD3DLight9));
  light._Type       := D3DLIGHT_DIRECTIONAL;
  light.Diffuse.r   := 1.0;
  light.Diffuse.g   := 1.0;
  light.Diffuse.b   := 1.0;
  D3DXVec3Normalize(light.Direction, vecLightDirUnnormalized);
  light.Position.x   := 10.0;
  light.Position.y   := -10.0;
  light.Position.z   := 10.0;
  light.Range        := 1000.0;
  m_pd3dDevice.SetLight(0, light);
  m_pd3dDevice.LightEnable(0, True);

  // Set the transform matrices
  D3DXMatrixIdentity(matWorld);
  m_pd3dDevice.SetTransform(D3DTS_WORLD, matWorld);

  // Setup the camera with view & projection matrix
  vecEye := D3DXVector3(0.0,-5.0,-10.0);
  vecAt  := D3DXVector3(0.2, 0.0, 0.0);
  m_Camera.SetViewParams(vecEye, vecAt);
  fAspectRatio := m_d3dsdBackBuffer.Width / m_d3dsdBackBuffer.Height;
  m_Camera.SetProjParams(D3DX_PI/4, fAspectRatio, 1.0, 1000.0);

  SAFE_RELEASE(m_pMesh3DText);
  if FAILED(CreateD3DXTextMesh(m_pMesh3DText, m_strFont, m_nFontSize, False, False)) then
  begin
    Result:= E_FAIL;
    Exit;
  end;

  Result:= S_OK;
end;


const
  IsBold: array[False..True] of Integer = (FW_NORMAL, FW_BOLD);

//-----------------------------------------------------------------------------
// Name: CreateD3DXTextMesh()
// Desc:
//-----------------------------------------------------------------------------
function CMyD3DApplication.CreateD3DXTextMesh(out ppMesh: ID3DXMesh; pstrFont: PChar; dwSize: DWORD; bBold, bItalic: Boolean): HRESULT;
var
  pMeshNew: ID3DXMesh;
  hdc: Windows.HDC;
  nHeight: Integer;
  hFont: Windows.HFONT;
  hFontOld: Windows.HFONT;
begin
  hdc := CreateCompatibleDC(0);
  nHeight := -MulDiv(dwSize, GetDeviceCaps(hdc, LOGPIXELSY), 72);

  hFont := CreateFont(nHeight, 0, 0, 0, IsBold[bBold], Integer(bItalic), iFalse, iFalse,
                      DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY,
                      DEFAULT_PITCH or FF_DONTCARE, pstrFont);

  hFontOld := SelectObject(hdc, hFont);

  Result := D3DXCreateText(m_pd3dDevice, hdc, 'This is calling D3DXCreateText',
                      0.001, 0.4, pMeshNew, nil, nil);

  SelectObject(hdc, hFontOld);
  DeleteObject(hFont);
  DeleteDC(hdc);

  if SUCCEEDED(Result) then ppMesh := pMeshNew;
end;


//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove: HRESULT;
var
  vAxis1, vAxis2: TD3DXVector3;
  mScale: TD3DXMatrix;
begin
  // Setup five rotation matrices (for rotating text strings)
  vAxis1 := D3DXVector3(1.0,2.0,0.0);
  vAxis2 := D3DXVector3(1.0,0.0,0.0);
  D3DXMatrixRotationAxis(m_matObj1, vAxis1, m_fTime/2.0);
  D3DXMatrixRotationAxis(m_matObj2, vAxis2, m_fTime);

  D3DXMatrixScaling(mScale, 0.5, 0.5, 0.5);
  D3DXMatrixMultiply(m_matObj2, m_matObj2, mScale);

  // Add some translational values to the matrices
  m_matObj1._41 := 1.0;   m_matObj1._42 := 6.0;   m_matObj1._43 := 20.0;
  m_matObj2._41 := -4.0;  m_matObj2._42 := -1.0;  m_matObj2._43 := 0.0;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render: HRESULT;
var
  rc: TRect;
  mtrl: TD3DMaterial9;
  matView, matProj, matViewProj: TD3DXMatrixA16;
begin
  // Get the view & projection matrix from camera.
  // User can't control camera for this simple sample
  matView := m_Camera.GetViewMatrix^;
  matProj := m_Camera.GetProjMatrix^;
  D3DXMatrixMultiply(matViewProj, matView, matProj);

  m_pd3dDevice.SetTransform(D3DTS_VIEW,       matView);
  m_pd3dDevice.SetTransform(D3DTS_PROJECTION, matProj);

  // Clear the viewport
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER, $00000000, 1.0, 0);

  // Begin the scene 
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    // Demonstration 1:
    // Draw a simple line using ID3DXFont::DrawText
{$IFNDEF SomeNeverDefinedSymbol}
    // Pass in DT_NOCLIP so we don't have to calc the bottom/right of the rect
    SetRect(rc, 150, 200, 0, 0);
    m_pFont.DrawTextA(nil, 'This is a trival call to ID3DXFont::DrawText', -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.0, 0.0, 1.0)));
{$ELSE}
    // If you wanted to calc the bottom/rect of the rect make these 2 calls
    SetRect(rc, 150, 200, 0, 0);
    m_pFont.DrawTextA(nil, 'This is a trival call to ID3DXFont::DrawText', -1, @rc, DT_CALCRECT, D3DXColorToDWord(D3DXColor(1.0, 0.0, 0.0, 1.0)));
    m_pFont.DrawTextA(nil, 'This is a trival call to ID3DXFont::DrawText', -1, @rc, 0,           D3DXColorToDWord(D3DXColor(1.0, 0.0, 0.0, 1.0)));
{$ENDIF}

    // Demonstration 2:
    // Allow multiple draw calls to sort by texture changes by ID3DXSprite
    // When drawing 2D text use flags: D3DXSPRITE_ALPHABLEND | D3DXSPRITE_SORT_TEXTURE
    // When drawing 3D text use flags: D3DXSPRITE_ALPHABLEND | D3DXSPRITE_SORT_BACKTOFRONT
    m_pD3DXSprite._Begin(D3DXSPRITE_ALPHABLEND or D3DXSPRITE_SORT_TEXTURE);
    SetRect(rc, 10, m_d3dpp.BackBufferHeight - 15*6, 0, 0);
    m_pFont2.DrawTextA(m_pD3DXSprite, 'These multiple calls to ID3DXFont.DrawText() using the same ID3DXSprite.', -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    SetRect(rc, 10, m_d3dpp.BackBufferHeight - 15*5, 0, 0);
    m_pFont2.DrawTextA(m_pD3DXSprite, 'ID3DXFont now caches letters on one or more textures.', -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    SetRect(rc, 10, m_d3dpp.BackBufferHeight - 15*4, 0, 0);
    m_pFont2.DrawTextA(m_pD3DXSprite, 'In order to sort by texture state changes on muliple',  -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    SetRect(rc, 10, m_d3dpp.BackBufferHeight - 15*3, 0, 0);
    m_pFont2.DrawTextA(m_pD3DXSprite, 'draw calls to DrawText() pass a ID3DXSprite and use',   -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    SetRect(rc, 10, m_d3dpp.BackBufferHeight - 15*2, 0, 0);
    m_pFont2.DrawTextA(m_pD3DXSprite, 'flags D3DXSPRITE_ALPHABLEND | D3DXSPRITE_SORT_TEXTURE', -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    m_pD3DXSprite._End;

    // Demonstration 3:
    // Word wrapping and unicode text.  
    // Note that not all fonts support dynamic font linking. 
    rc.left := 10;
    rc.top := 60;
    rc.right := m_d3dpp.BackBufferWidth - 10;
    rc.bottom := m_d3dpp.BackBufferHeight - 10;
    m_pFont2.DrawTextW(nil, m_strTextBuffer, -1, @rc, DT_LEFT or DT_WORDBREAK or DT_EXPANDTABS, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));

    // Demonstration 4:
    // Draw 3D extruded text using a mesh created with D3DXFont
    if (m_pMesh3DText <> nil) then
    begin
      ZeroMemory(@mtrl, SizeOf(TD3DMaterial9));
      mtrl.Diffuse.r := 0.0; mtrl.Ambient.r := 0.0;
      mtrl.Diffuse.g := 0.0; mtrl.Ambient.g := 0.0;
      mtrl.Diffuse.b := 0.0; mtrl.Ambient.b := 1.0;
      mtrl.Diffuse.a := 1.0; mtrl.Ambient.a := 1.0;
      m_pd3dDevice.SetMaterial(mtrl);
      m_pd3dDevice.SetTransform(D3DTS_WORLD, m_matObj2);
      m_pMesh3DText.DrawSubset(0);
    end;

    // Show frame rate
    SetRect(rc, 2, 0, 0, 0);
    m_pStatsFont.DrawTextA(nil, m_strFrameStats, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));
    SetRect(rc, 2, 15, 0, 0);
    m_pStatsFont.DrawTextA(nil, m_strDeviceStats, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));

    // End the scene
    m_pd3dDevice.EndScene;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects: HRESULT;
begin
  m_pStatsFont.OnLostDevice;
  m_pFont.OnLostDevice;
  m_pFont2.OnLostDevice;
  m_pD3DXSprite.OnLostDevice;
  SAFE_RELEASE(m_pMesh3DText);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects: HRESULT;
begin
  SAFE_RELEASE(m_pFont);
  SAFE_RELEASE(m_pFont2);
  SAFE_RELEASE(m_pStatsFont);
  SAFE_RELEASE(m_pD3DXSprite);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup: HRESULT;
begin
  FreeMem(m_strTextBuffer);
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Message proc function to handle key and menu input
//-----------------------------------------------------------------------------
function CMyD3DApplication.MsgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM ): LRESULT;
var
  hr: HRESULT;
  hdc: Windows.HDC;
  lHeight: Longint;
  lf: TLogFont;
  cf: TChooseFont;

  pFontNew: ID3DXFont;
  pMesh3DTextNew: ID3DXMesh;
  pstrFontNameNew: PChar;
  dwFontSizeNew: Integer;
  bSuccess, bBold, bItalic: Boolean;
  nLogPixelsY: Integer;
  nHeight: Integer;
begin
  case uMsg of
    WM_SIZE:
    begin
      if (SIZE_RESTORED = wParam) then
      begin
        if (not m_bMaximized and not m_bMinimized) then
        begin
          // This sample demonstrates word wrapping, so if the
          // window size is changing because the user dragging the window
          // edges we'll recalc the size, re-init, and repaint
          // the scene as the window resizes.  This would be very
          // slow for complex scene but works here since this sample
          // is trival.
          HandlePossibleSizeChange;

          // Repaint the window
          if (Assigned(m_pd3dDevice) and not m_bActive and m_bWindowed and
              m_bDeviceObjectsInited and m_bDeviceObjectsRestored) then
          begin
            Render;
            hr := m_pd3dDevice.Present(nil, nil, 0, nil);
            if (D3DERR_DEVICELOST = hr) then m_bDeviceLost := True;
          end;
        end;
      end;
    end;

    WM_COMMAND:
    begin
      case LOWORD(wParam) of
        IDM_FONT:
        begin
          hdc := GetDC(hWnd);
          lHeight := -MulDiv(m_nFontSize, GetDeviceCaps(hdc, LOGPIXELSY), 72);
          ReleaseDC(hWnd, hdc);
          // hdc := 0; // never used anyway

          StrCopy(lf.lfFaceName, m_strFont);
          lf.lfHeight := lHeight;

          ZeroMemory(@cf, SizeOf(cf));
          cf.lStructSize := SizeOf(cf);
          cf.hwndOwner   := m_hWnd;
          cf.lpLogFont   := @lf;
          cf.Flags       := CF_SCREENFONTS or CF_INITTOLOGFONTSTRUCT or CF_TTONLY;

          if ChooseFont(cf) then
          begin
            pFontNew := nil;
            pMesh3DTextNew := nil;
            pstrFontNameNew := lf.lfFaceName;
            dwFontSizeNew := cf.iPointSize div 10;
            bSuccess := False;
            bBold   := ((cf.nFontType and BOLD_FONTTYPE) = BOLD_FONTTYPE);
            bItalic := ((cf.nFontType and ITALIC_FONTTYPE) = ITALIC_FONTTYPE);

            hDC := GetDC(0);
            nLogPixelsY := GetDeviceCaps(hDC, LOGPIXELSY);
            ReleaseDC(0, hDC);

            nHeight := -MulDiv(dwFontSizeNew, nLogPixelsY, 72);
            if SUCCEEDED(D3DXCreateFont(m_pd3dDevice, nHeight, 0, IsBold[bBold], 0, bItalic, DEFAULT_CHARSET,
                           OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH or FF_DONTCARE, pstrFontNameNew, pFontNew)) then
            begin
              if SUCCEEDED(CreateD3DXTextMesh(pMesh3DTextNew, pstrFontNameNew, dwFontSizeNew, bBold, bItalic)) then
              begin
                bSuccess := True;
                SAFE_RELEASE(m_pFont);
                m_pFont := pFontNew;
                pFontNew := nil;

                SAFE_RELEASE(m_pMesh3DText);
                m_pMesh3DText := pMesh3DTextNew;
                pMesh3DTextNew := nil;

                StrCopy(m_strFont, pstrFontNameNew);
                m_nFontSize := dwFontSizeNew;
              end;
            end;

            SAFE_RELEASE(pMesh3DTextNew);
            SAFE_RELEASE(pFontNew);

            if not bSuccess
            then MessageBox(m_hWnd, 'Could not create that font.', m_strWindowTitle, MB_OK );
          end;
        end;
      end;
    end;
  end;

  Result := inherited MsgProc(hWnd, uMsg, wParam, lParam);
end;


end.

